@@ -2,7 +2,8 @@ package Archive::Libarchive::FFI;
 
 use strict;
 use warnings;
-use Alien::Libarchive::Installer;
+use Alien::Libarchive;
+use I18N::Langinfo ();
 use Exporter::Tidy ();
 use Encode ();
 use Carp qw( croak );
@@ -18,6 +19,7 @@ use FFI::Util qw(
   scalar_to_buffer
   deref_int_get
   deref_str_get
+  locate_module_share_lib
   :types
 );
 
@@ -32,15 +34,20 @@ BEGIN {
     require Archive::Libarchive::FFI::SweetLite;
     Archive::Libarchive::FFI::SweetLite->import;
   }
-
 }
 
 # ABSTRACT: Perl bindings to libarchive via FFI
-our $VERSION = '0.0801_01'; # VERSION
+our $VERSION = '0.04'; # VERSION
 
-ffi_lib(\$_) for Alien::Libarchive::Installer->system_install( test => 'ffi' )->dlls;
+ffi_lib do {
+  my $file = locate_module_share_lib();
+  \$file;
+};
+
+ffi_lib(Alien::Libarchive->new);
 
 require Archive::Libarchive::FFI::Constant;
+require Archive::Libarchive::FFI::Callback;
 
 $Archive::Libarchive::FFI::on_attach ||= sub {};
 
@@ -56,17 +63,7 @@ sub _attach ($$$)
 {
   $Archive::Libarchive::FFI::on_attach->(@_);
   my($name, $arg, $ret) = @_;
-  if(grep { $_ == FFI::Raw::str } ($ret, @$arg))
-  {
-    if(ref $name)
-    {
-      $name->[1] = "_" . $name->[1];
-    }
-    else
-    {
-      $name = [ $name => "_$name" ] 
-    }
-  }
+  $name = [ $name => "_$name" ] if grep { $_ == FFI::Raw::str } ($ret, @$arg);
   if($ret == _void)
   {
     _attach_function $name, $arg, $ret, sub {
@@ -82,9 +79,6 @@ sub _attach ($$$)
 }
 
 _attach 'archive_version_number',                        undef, _int;
-
-require Archive::Libarchive::FFI::Callback;
-
 _attach 'archive_version_string',                        undef, _str;
 _attach 'archive_clear_error',                           [ _ptr ], _void;
 _attach 'archive_copy_error',                            [ _ptr ], _int;
@@ -93,22 +87,10 @@ _attach 'archive_file_count',                            [ _ptr ], _int;
 _attach 'archive_format',                                [ _ptr ], _int;
 _attach 'archive_format_name',                           [ _ptr ], _str;
 _attach 'archive_seek_data',                             [ _ptr, _int64, _int ], _int64;
-
-if(archive_version_number() >= 3000000)
-{
-  _attach 'archive_error_string',                          [ _ptr ], _str;
-}
-else
-{
-  _attach_function [ 'archive_error_string' => '_archive_error_string' ], [ _ptr ], _str, sub {
-    my($sub, $archive) = @_;
-    my $ret = $sub->($archive);
-    return if $ret eq '(Empty error message)';
-    return $ret;
-  };
-}
+_attach 'archive_error_string',                          [ _ptr ], _str;
 
 _attach 'archive_read_new',                              undef, _ptr;
+_attach 'archive_read_support_filter_all',               [ _ptr ], _int;
 _attach 'archive_read_support_format_all',               [ _ptr ], _int;
 _attach 'archive_read_open1',                            [ _ptr ], _int;
 _attach 'archive_read_open_filename',                    [ _ptr, _str, _int ], _int;
@@ -116,6 +98,7 @@ _attach 'archive_read_data_skip',                        [ _ptr ], _int;
 _attach 'archive_read_close',                            [ _ptr ], _int;
 _attach 'archive_read_append_filter',                    [ _ptr, _int ], _int;
 _attach 'archive_read_append_filter_program',            [ _ptr, _str ], _int;
+_attach 'archive_read_support_filter_program',           [ _ptr, _str ], _int;
 _attach 'archive_read_support_format_by_code',           [ _ptr, _int ], _int;
 _attach 'archive_read_header_position',                  [ _ptr ], _int64;
 _attach 'archive_read_set_filter_option',                [ _ptr, _str, _str, _str ], _int;
@@ -128,16 +111,6 @@ _attach 'archive_read_extract',                          [ _ptr, _ptr, _int ], _
 _attach 'archive_read_extract2',                         [ _ptr, _ptr, _ptr ], _int;
 _attach 'archive_read_extract_set_skip_file',            [ _ptr, _int64, _int64 ], _void;
 
-if(archive_version_number() >= 3000000)
-{
-  _attach 'archive_read_support_filter_program', [ _ptr, _str ], _int;
-}
-else
-{
-  _attach ['archive_read_support_compression_program' => 'archive_read_support_filter_program'], [ _ptr, _str ], _int;
-}
-
-
 _attach 'archive_filter_code',                           [ _ptr, _int ], _int;
 _attach 'archive_filter_count',                          [ _ptr ], _int;
 _attach 'archive_filter_name',                           [ _ptr, _int ], _str;
@@ -146,6 +119,7 @@ _attach 'archive_filter_bytes',                          [ _ptr, _int ], _int64;
 _attach 'archive_write_new',                             undef, _ptr;
 _attach 'archive_write_add_filter',                      [ _ptr, _int ], _int;
 _attach 'archive_write_add_filter_by_name',              [ _ptr, _str ], _int;
+_attach 'archive_write_add_filter_program',              [ _ptr, _str ], _int;
 _attach 'archive_write_set_format',                      [ _ptr, _int ], _int;
 _attach 'archive_write_set_format_by_name',              [ _ptr, _str ], _int;
 _attach 'archive_write_open_filename',                   [ _ptr, _str ], _int;
@@ -171,16 +145,6 @@ _attach 'archive_write_get_bytes_per_block',             [ _ptr ], _int;
 _attach 'archive_write_set_bytes_in_last_block',         [ _ptr, _int ], _int;
 _attach 'archive_write_set_bytes_per_block',             [ _ptr, _int ], _int;
 
-if(archive_version_number() >= 3000000)
-{
-  _attach 'archive_write_add_filter_program', [ _ptr, _str ], _int;
-}
-else
-{
-  _attach ['archive_write_set_compression_program' => 'archive_write_add_filter_program'], [ _ptr, _str ], _int;
-}
-
-
 _attach 'archive_entry_clear',                           [ _ptr ], _void;
 _attach 'archive_entry_clone',                           [ _ptr ], _ptr;
 _attach 'archive_entry_free',                            [ _ptr ], _void;
@@ -211,7 +175,7 @@ _attach 'archive_entry_dev',                             [ _ptr ], _dev_t;
 _attach 'archive_entry_devmajor',                        [ _ptr ], _dev_t;
 _attach 'archive_entry_devminor',                        [ _ptr ], _dev_t;
 _attach 'archive_entry_fflags_text',                     [ _ptr ], _str;
-_attach 'archive_entry_gid',                             [ _ptr ], _int64;
+_attach 'archive_entry_gid',                             [ _ptr ], _gid_t;
 _attach 'archive_entry_rdev',                            [ _ptr ], _int64;
 _attach 'archive_entry_rdevmajor',                       [ _ptr ], _int64;
 _attach 'archive_entry_rdevminor',                       [ _ptr ], _int64;
@@ -228,7 +192,7 @@ _attach 'archive_entry_set_dev',                         [ _ptr, _dev_t ], _void
 _attach 'archive_entry_set_devmajor',                    [ _ptr, _dev_t ], _void;
 _attach 'archive_entry_set_devminor',                    [ _ptr, _dev_t ], _void;
 _attach 'archive_entry_set_fflags',                      [ _ptr, _ulong, _ulong ], _void;
-_attach 'archive_entry_set_gid',                         [ _ptr, _int64 ], _void;
+_attach 'archive_entry_set_gid',                         [ _ptr, _gid_t ], _void;
 _attach 'archive_entry_set_ino',                         [ _ptr, _int64 ], _void;
 _attach 'archive_entry_set_link',                        [ _ptr, _str ], _void;
 _attach 'archive_entry_set_mode',                        [ _ptr, _int ], _void;
@@ -243,10 +207,11 @@ _attach 'archive_entry_unset_size',                      [ _ptr ], _void;
 _attach 'archive_entry_xattr_clear',                     [ _ptr ], _void;
 _attach 'archive_entry_xattr_count',                     [ _ptr ], _int;
 _attach 'archive_entry_xattr_reset',                     [ _ptr ], _int;
-_attach 'archive_entry_uid',                             [ _ptr ], _int64;
+_attach 'archive_entry_uid',                             [ _ptr ], _uid_t;
 _attach 'archive_entry_copy_sourcepath',                 [ _ptr, _str ], _void;
 _attach 'archive_entry_acl',                             [ _ptr ], _ptr;
 _attach 'archive_entry_acl_clear',                       [ _ptr ], _int;
+_attach 'archive_entry_acl_add_entry',                   [ _ptr, _int, _int, _int, _int, _str ], _int;
 _attach 'archive_entry_acl_reset',                       [ _ptr, _int ], _int;
 _attach 'archive_entry_acl_text',                        [ _ptr, _int ], _str;
 _attach 'archive_entry_acl_count',                       [ _ptr, _int ], _int;
@@ -255,18 +220,6 @@ _attach 'archive_entry_sparse_add_entry',                [ _ptr, _int64, _int64
 _attach 'archive_entry_sparse_count',                    [ _ptr ], _int;
 _attach 'archive_entry_sparse_reset',                    [ _ptr ], _int;
 
-if(archive_version_number() >= 3000000)
-{
-  _attach 'archive_entry_acl_add_entry',                   [ _ptr, _int, _int, _int, _int, _str ], _int;
-}
-else
-{
-  _attach_function [ 'archive_entry_acl_add_entry' => '_archive_entry_acl_add_entry' ], [ _ptr, _int, _int, _int, _int, _str ], _void, sub {
-    shift->(@_);
-    ARCHIVE_OK();
-  };
-}
-
 _attach 'archive_entry_linkresolver_free',               [ _ptr ], _void;
 _attach 'archive_entry_linkresolver_new',                undef, _ptr;
 _attach 'archive_entry_linkresolver_set_strategy',       [ _ptr, _int ], _void;
@@ -293,8 +246,8 @@ _attach 'archive_match_excluded',                        [ _ptr, _ptr ], _int;
 _attach 'archive_match_path_excluded',                   [ _ptr, _ptr ], _int;
 _attach 'archive_match_time_excluded',                   [ _ptr, _ptr ], _int;
 _attach 'archive_match_owner_excluded',                  [ _ptr, _ptr ], _int;
-_attach 'archive_match_include_gid',                     [ _ptr, _int64 ], _int;
-_attach 'archive_match_include_uid',                     [ _ptr, _int64 ], _int;
+_attach 'archive_match_include_gid',                     [ _ptr, _gid_t ], _int;
+_attach 'archive_match_include_uid',                     [ _ptr, _uid_t ], _int;
 _attach 'archive_match_include_gname',                   [ _ptr, _str ], _int;
 _attach 'archive_match_include_uname',                   [ _ptr, _str ], _int;
 _attach 'archive_match_exclude_entry',                   [ _ptr, _int, _ptr ], _int;
@@ -306,35 +259,12 @@ _attach 'archive_match_include_file_time',               [ _ptr, _int, _str ], _
 _attach 'archive_match_include_time',                    [ _ptr, _int, _time_t, _long ], _int;
 _attach 'archive_match_path_unmatched_inclusions',       [ _ptr ], _int;
 
-foreach my $type (qw( all bzip2 compress gzip grzip lrzip lzip lzma lzop none rpm uu xz ))
-{
-  my $name = "archive_read_support_filter_$type";
-  eval {
-    attach_function $name, [ _ptr ], _int;
-  };
-  if($@)
-  {
-    my $real = "archive_read_support_compression_$type";
-    eval { attach_function [ $real => $name ], [ _ptr ], _int };
-  }
-}
-
+_attach "archive_read_support_filter_$_",  [ _ptr ], _int
+  for qw( bzip2 compress gzip grzip lrzip lzip lzma lzop none rpm uu xz );
 _attach "archive_read_support_format_$_",  [ _ptr ], _int
   for qw( 7zip ar cab cpio empty gnutar iso9660 lha mtree rar raw tar xar zip );
-
-foreach my $type (qw( b64encode bzip2 compress grzip gzip lrzip lzip lzma lzop none uuencode xz ))
-{
-  my $name = "archive_write_add_filter_$type";
-  eval {
-    attach_function $name, [ _ptr ], _int;
-  };
-  if($@)
-  {
-    my $real = "archive_write_set_compression_$type";
-    eval { attach_function [ $real => $name ], [ _ptr ], _int };
-  }
-}  
-
+_attach "archive_write_add_filter_$_", [ _ptr ], _int
+  for qw( b64encode bzip2 compress grzip gzip lrzip lzip lzma lzop none uuencode xz );
 _attach "archive_write_set_format_$_", [ _ptr ], _int
   for qw( 7zip ar_bsd ar_svr4 cpio cpio_newc gnutar iso9660 mtree mtree_classic 
           pax pax_restricted shar shar_dump ustar v7tar xar zip);
@@ -482,7 +412,7 @@ _attach_function 'archive_entry_xattr_next', [ _ptr, _ptr, _ptr, _ptr ], _int, s
   
   my $ret = $_[0]->($_[1], $name, $ptr, $size);
   $_[2] = deref_str_get($$name);
-  $_[3] = buffer_to_scalar(deref_ptr_get($$ptr), deref_size_t_get($$size));
+  $_[3] = buffer_to_scalar(deref_ptr_get($$ptr), deref_int64_get($$size));
   
   $ret;
 };
@@ -494,7 +424,7 @@ _attach_function 'archive_entry_mac_metadata', [ _ptr, _ptr ], _ptr, sub
   my($cb, $archive) = @_;
   my $size = FFI::Raw::MemPtr->new_from_ptr(0);
   my $ptr = $cb->($archive, $size);
-  my $buffer = buffer_to_scalar($ptr, deref_size_t_get($$size));
+  my $buffer = buffer_to_scalar($ptr, deref_uint64_get($$size));
 };
 
 _attach_function 'archive_set_error', [ _ptr, _int, _str, _str ], _void, sub
@@ -517,15 +447,7 @@ _attach_function $_, [ _ptr, _str, _ptr, _size_t ],_int, sub
 {
   my($cb, $archive, $command, $signature) = @_;
   $cb->($archive, $command, scalar_to_buffer($signature));
-} for (
-  'archive_read_append_filter_program_signature',
-# TODO:
-# using archive_read_support_compression_program_signature for archive_read_support_filter_program_signature doesn't
-# appear to actually work for libarchive 2.x, so I am commenting it out for now.  This does appear to make it slightly
-# out of sync with the capability of the XS version, so if you need this function either use the XS version, or
-# upgrade to libarchie >= 3
-#  archive_version_number() >= 3000000 ? 'archive_read_support_filter_program_signature' : [ archive_read_support_compression_program_signature => 'archive_read_support_filter_program_signature']);
-  'archive_read_support_filter_program_signature');
+} for qw( archive_read_append_filter_program_signature archive_read_support_filter_program_signature );
 
 # this is an unusual one which doesn't need to be decoded
 # because it should always be ASCII
@@ -543,8 +465,7 @@ _attach_function 'archive_entry_linkify', [ _ptr, _ptr, _ptr ], _void, sub
 
 _attach_function [ 'archive_entry_copy_fflags_text' => '_archive_entry_set_fflags_text' ], [ _ptr, _str ], _void, sub
 {
-  my($sub, $entry, $text) = @_;
-  $sub->($entry, $text);
+  shift->(@_);
   ARCHIVE_OK();
 };
 
@@ -558,37 +479,51 @@ _attach_function 'archive_read_disk_entry_from_file', [ _ptr, _ptr, _int, _ptr ]
   $cb->($archive, $entry, $fd, 0);
 };
 
-if(eval q{ require I18N::Langinfo; 1 })
+_attach_function [ 'my_set_user_data_name' => '_my_set_user_data_name' ], [ _ptr, _str ], _void;
+
+foreach my $type (qw( uname gname ))
 {
-  eval '# line '. __LINE__ . ' "' . __FILE__ . qq("\n) . q{
-    sub archive_perl_codeset
-    {
-      I18N::Langinfo::langinfo(I18N::Langinfo::CODESET());
-    }
-    sub archive_perl_utf8_mode
-    {
-      int(I18N::Langinfo::langinfo(I18N::Langinfo::CODESET()) eq 'UTF-8');
-    }
+  _attach_function [ "my_archive_read_disk_set_$type\_lookup" => "archive_read_disk_set_$type\_lookup" ], [ _ptr, _ptr, _ptr ], _int, sub
+  {
+    my($cb, $archive, $data, $lookup, $cleanup) = @_;
+
+    # closures are fun.
+    # er.    
+    my $lookup2;
+    my $cleanup2;
+    
+    $lookup2 =  FFI::Raw::Callback->new(sub {
+      my($ptr, $id) = @_;
+      if($lookup)
+      {
+        my $str = $lookup->($data, $id);
+        _my_set_user_data_name($ptr, $str) if defined $str;
+      }
+    }, _void, _ptr, _int64);
+    
+    $cleanup2 = FFI::Raw::Callback->new(sub {
+      $cleanup->($data) if $cleanup;
+      undef $lookup2;
+      undef $cleanup2;
+    }, _void);
+    
+    $cb->($archive, $lookup2, $cleanup2);
   };
-  die $@ if $@;
 }
-else
+
+sub archive_perl_codeset
 {
-  eval '# line '. __LINE__ . ' "' . __FILE__ . qq("\n) . q{
-    sub archive_perl_codeset
-    {
-      'ANSI_X3.4-1968';
-    }
-    sub archive_perl_utf8_mode
-    {
-      0;
-    }
-  };  
+  I18N::Langinfo::langinfo(I18N::Langinfo::CODESET());
+}
+
+sub archive_perl_utf8_mode
+{
+  int(I18N::Langinfo::langinfo(I18N::Langinfo::CODESET()) eq 'UTF-8');
 }
 
 require Archive::Libarchive::FFI::Common;
 
-eval '# line '. __LINE__ . ' "' . __FILE__ . qq("\n) . q{
+eval q{
   use Exporter::Tidy
     func  => [grep /^archive_/,       keys %Archive::Libarchive::FFI::],
     const => [grep /^(AE_|ARCHIVE_)/, keys %Archive::Libarchive::FFI::];
@@ -608,7 +543,7 @@ Archive::Libarchive::FFI - Perl bindings to libarchive via FFI
 
 =head1 VERSION
 
-version 0.0801_01
+version 0.04
 
 =head1 SYNOPSIS
 
@@ -1157,15 +1092,6 @@ read:
  # or
  my $decoded_content = decode('KOI8-R', $raw);
 
-=head1 SUPPORT
-
-If you find bugs, please open an issue on the project GitHub repository:
-
-L<https://github.com/plicease/Archive-Libarchive-FFI/issues?state=open>
-
-If you have a fix, please open a pull request.  You can see the CONTRIBUTING
-file for traps, hints and pitfalls.
-
 =head1 CAVEATS
 
 Archive and entry objects are really pointers to opaque C structures
