@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 # ABSTRACT: Libarchive callbacks
-our $VERSION = '0.0801_01'; # VERSION
+our $VERSION = '0.04'; # VERSION
 
 package
   Archive::Libarchive::FFI;
@@ -37,17 +37,6 @@ use constant {
 
 my %callbacks;
 
-do {
-  no warnings 'redefine';
-  sub _attach_function ($$$;$)
-  {
-    eval {
-      attach_function($_[0], $_[1], $_[2], $_[3]);
-    };
-    warn $@ if $@ && $ENV{ARCHIVE_LIBARCHIVE_FFI_VERBOSE};
-  }
-};
-
 my $myopen = FFI::Raw::Callback->new(sub {
   my($archive) = @_;
   my $status = eval {
@@ -74,7 +63,7 @@ my $mywrite = FFI::Raw::Callback->new(sub
     return ARCHIVE_FATAL();
   }
   $status;
-}, _int, _ptr, _ptr, _ptr, _size_t);
+}, _int, _ptr, _ptr, _ptr, _int64);
 
 my $myread = FFI::Raw::Callback->new(sub
 {
@@ -134,7 +123,7 @@ my $myclose = FFI::Raw::Callback->new(sub
   $status;
 }, _int, _ptr, _ptr);
 
-_attach_function 'archive_write_open', [ _ptr, _ptr, _ptr, _ptr, _ptr ], _int, sub
+attach_function 'archive_write_open', [ _ptr, _ptr, _ptr, _ptr, _ptr ], _int, sub
 {
   my($cb, $archive, $cd, $open, $write, $close) = @_;
   $callbacks{$archive}->[CB_DATA] = $cd;
@@ -153,7 +142,7 @@ _attach_function 'archive_write_open', [ _ptr, _ptr, _ptr, _ptr, _ptr ], _int, s
     $callbacks{$archive}->[CB_CLOSE] = $close;
     $close = $myclose;
   }
-  $cb->($archive, undef, $open||0, $write||0, $close||0);
+  $cb->($archive, 0, $open||0, $write||0, $close||0);
 };
 
 sub archive_read_open ($$$$$)
@@ -162,7 +151,7 @@ sub archive_read_open ($$$$$)
   archive_read_open2($archive, $data, $open, $read, undef, $close);
 }
 
-_attach_function 'archive_read_open2', [ _ptr, _ptr, _ptr, _ptr, _ptr, _ptr ], _int, sub
+attach_function 'archive_read_open2', [ _ptr, _ptr, _ptr, _ptr, _ptr, _ptr ], _int, sub
 {
   my($cb, $archive, $cd, $open, $read, $skip, $close) = @_;
   $callbacks{$archive}->[CB_DATA] = $cd;
@@ -186,7 +175,7 @@ _attach_function 'archive_read_open2', [ _ptr, _ptr, _ptr, _ptr, _ptr, _ptr ], _
     $callbacks{$archive}->[CB_CLOSE] = $close;
     $close = $myclose;
   }
-  $cb->($archive, undef, $open||0, $read||0, $skip||0, $close||0);
+  $cb->($archive, 0, $open||0, $read||0, $skip||0, $close||0);
 };
 
 sub archive_read_set_callback_data ($$)
@@ -208,48 +197,19 @@ foreach my $name (qw( open read skip close seek ))
     }
   };die $@ if $@;
   
-  _attach_function "archive_read_set_$name\_callback", [ _ptr, _ptr ], _int;
+  attach_function "archive_read_set_$name\_callback", [ _ptr, _ptr ], _int;
 }
 
-if(archive_version_number() >= 3000000)
+attach_function 'archive_read_open_memory', [ _ptr, _ptr, _size_t ], _int, sub
 {
-  _attach_function 'archive_read_open_memory', [ _ptr, _ptr, _size_t ], _int, sub
-  {
-    my($cb, $archive, $buffer) = @_;
-    my $length = do { use bytes; length $buffer };
-    my $ptr = FFI::Raw::MemPtr->new_from_buf($buffer, $length);
-    $callbacks{$archive}->[CB_BUFFER] = $ptr;  # TODO: CB_BUFFER or CB_DATA (or something else?)
-    $cb->($archive, $ptr, $length);
-  };
-}
-else
-{
-  sub _archive_read_open_memory_read
-  {
-    my($archive, $data) = @_;
-    if($data->{done})
-    {
-      return (ARCHIVE_OK(), '');
-    }
-    else
-    {
-      $data->{done} = 1;
-      return (ARCHIVE_OK(), $data->{buffer});
-    }
-  }
+  my($cb, $archive, $buffer) = @_;
+  my $length = do { use bytes; length $buffer };
+  my $ptr = FFI::Raw::MemPtr->new_from_buf($buffer, $length);
+  $callbacks{$archive}->[CB_BUFFER] = $ptr;  # TODO: CB_BUFFER or CB_DATA (or something else?)
+  $cb->($archive, $ptr, $length);
+};
 
-  *archive_read_open_memory = sub ($$) {
-    my($archive, $buffer) = @_;
-    my $r = archive_read_open($archive, { buffer => $buffer, done => 0 }, undef, \&_archive_read_open_memory_read, undef);
-    unless($r == ARCHIVE_OK())
-    {
-      warn "error: " . archive_error_string($archive);
-    }
-    $r;
-  };
-}
-
-_attach_function archive_version_number() >= 3000000 ? 'archive_read_free' : [ archive_read_finish => 'archive_read_free' ], [ _ptr ], _int, sub
+attach_function 'archive_read_free', [ _ptr ], _int, sub
 {
   my($cb, $archive) = @_;
   my $ret = $cb->($archive);
@@ -257,7 +217,7 @@ _attach_function archive_version_number() >= 3000000 ? 'archive_read_free' : [ a
   $ret;
 };
 
-_attach_function archive_version_number() >= 3000000 ? 'archive_write_free' : [ archive_write_finish => 'archive_write_free' ], [ _ptr ], _int, sub
+attach_function 'archive_write_free', [ _ptr ], _int, sub
 {
   my($cb, $archive) = @_;
   my $ret = $cb->($archive);
@@ -282,28 +242,11 @@ my $mylook_write_user_lookup = FFI::Raw::Callback->new(sub {
 my $mylook_write_group_lookup = FFI::Raw::Callback->new(sub {
   my($archive, $name, $id) = @_;
   my($data, $look_cb, $clean_cb) = @{ $lookups{$archive}->[CB_LOOKUP_GROUP] };
+  $DB::single = 1;
   return $id unless defined $look_cb;
   $look_cb->($data, $name, $id);
 }, _int64, _ptr, _str, _int64);
 
-my $mylook_read_user_lookup = FFI::Raw::Callback->new(sub {
-  my($archive, $id) = @_;
-  my($data, $look_cb, $clean_cb) = @{ $lookups{$archive}->[CB_LOOKUP_USER] };
-  return undef unless defined $look_cb;
-  my $name = $look_cb->($data, $id);
-  return $name if defined $name;
-  return;
-}, _str, _ptr, _int64);
-
-my $mylook_read_group_lookup = FFI::Raw::Callback->new(sub {
-  my($archive, $id) = @_;
-  my($data, $look_cb, $clean_cb) = @{ $lookups{$archive}->[CB_LOOKUP_GROUP] };
-  return undef unless defined $look_cb;
-  my $name = $look_cb->($data, $id);
-  return $name if defined $name;
-  return;
-}, _str, _ptr, _int64);
-
 my $mylook_user_cleanup = FFI::Raw::Callback->new(sub {
   my($archive) = @_;
   my($data, $look_cb, $clean_cb) = @{ $lookups{$archive}->[CB_LOOKUP_USER] };
@@ -314,11 +257,12 @@ my $mylook_user_cleanup = FFI::Raw::Callback->new(sub {
 my $mylook_group_cleanup = FFI::Raw::Callback->new(sub {
   my($archive) = @_;
   my($data, $look_cb, $clean_cb) = @{ $lookups{$archive}->[CB_LOOKUP_GROUP] };
+  $DB::single = 1;
   $clean_cb->($data) if defined $clean_cb;
   delete $lookups{$archive};
 }, _void, _ptr);
 
-_attach_function 'archive_write_disk_set_user_lookup', [ _ptr, _ptr, _ptr, _ptr ], _int, sub
+attach_function 'archive_write_disk_set_user_lookup', [ _ptr, _ptr, _ptr, _ptr ], _int, sub
 {
   my($cb, $archive, $data, $look_cb, $clean_cb) = @_;
   if(defined $look_cb || defined $clean_cb)
@@ -326,10 +270,10 @@ _attach_function 'archive_write_disk_set_user_lookup', [ _ptr, _ptr, _ptr, _ptr
     $lookups{$archive}->[CB_LOOKUP_USER] = [ $data, $look_cb, $clean_cb ];
     return $cb->($archive, $archive, $mylook_write_user_lookup, $mylook_user_cleanup);
   }
-  return $cb->($archive, undef, undef, undef);
+  return $cb->($archive,0,0,0);
 };
 
-_attach_function 'archive_write_disk_set_group_lookup', [ _ptr, _ptr, _ptr, _ptr ], _int, sub
+attach_function 'archive_write_disk_set_group_lookup', [ _ptr, _ptr, _ptr, _ptr ], _int, sub
 {
   my($cb, $archive, $data, $look_cb, $clean_cb) = @_;
   if(defined $look_cb || defined $clean_cb)
@@ -337,29 +281,7 @@ _attach_function 'archive_write_disk_set_group_lookup', [ _ptr, _ptr, _ptr, _ptr
     $lookups{$archive}->[CB_LOOKUP_GROUP] = [ $data, $look_cb, $clean_cb ];
     return $cb->($archive, $archive, $mylook_write_group_lookup, $mylook_group_cleanup);
   }
-  return $cb->($archive, undef, undef, undef);
-};
-
-_attach_function 'archive_read_disk_set_uname_lookup', [ _ptr, _ptr, _ptr, _ptr ], _int, sub
-{
-  my($cb, $archive, $data, $look_cb, $clean_cb) = @_;
-  if(defined $look_cb || defined $clean_cb)
-  {
-    $lookups{$archive}->[CB_LOOKUP_USER] = [ $data, $look_cb, $clean_cb ];
-    return $cb->($archive, $archive, $mylook_read_user_lookup, $mylook_user_cleanup);
-  }
-  return $cb->($archive, undef, undef, undef);
-};
-
-_attach_function 'archive_read_disk_set_gname_lookup', [ _ptr, _ptr, _ptr, _ptr ], _int, sub
-{
-  my($cb, $archive, $data, $look_cb, $clean_cb) = @_;
-  if(defined $look_cb || defined $clean_cb)
-  {
-    $lookups{$archive}->[CB_LOOKUP_GROUP] = [ $data, $look_cb, $clean_cb ];
-    return $cb->($archive, $archive, $mylook_read_group_lookup, $mylook_group_cleanup);
-  }
-  return $cb->($archive, undef, undef, undef);
+  return $cb->($archive,0,0,0);
 };
 
 1;
@@ -376,7 +298,7 @@ Archive::Libarchive::FFI::Callback - Libarchive callbacks
 
 =head1 VERSION
 
-version 0.0801_01
+version 0.04
 
 =head1 SYNOPSIS
 
@@ -470,7 +392,7 @@ L<archive_read_open2|Archive::Libarchive::FFI::Function#archive_read_open2>.
  my $mywrite = sub {
    my($archive, $data, $buffer) = @_;
    ...
-   return $bytes_written_or_status;
+   return $status1;
  };
  my $status2 = archive_write_open($archive, undef, $mywrite, undef);
 
@@ -478,9 +400,6 @@ This callback is called whenever libarchive has data it wants to send
 to output.  The callback itself takes one additional argument, a 
 buffer containing the data to write.
 
-It should return the actual number of bytes written by you, or an
-status value for an error.
-
 =head2 skip
 
  my $status1 = archive_read_set_skip_callback($archive, sub {
