@@ -20,7 +20,28 @@ my @libs;
 sub ffi_lib ($)
 {
   my $lib = shift;
-  push @libs, $$lib;
+  if(ref $lib)
+  {
+    if(eval { $lib->isa('Alien::Libarchive') })
+    {
+      if($lib->install_type eq 'share')
+      {
+        if($^O eq 'MSWin32')
+        {
+          die 'fixme';
+        }
+        elsif($^O eq 'cygwin')
+        {
+          die 'fixme';
+        }
+      }
+      push @libs, DynaLoader::dl_findfile(shellwords $lib->libs);
+    }
+    else
+    {
+      push @libs, $$lib;
+    }
+  }
 }
 
 sub attach_function ($$$;$)
@@ -74,7 +95,7 @@ Archive::Libarchive::FFI::SweetLite
 
 =head1 VERSION
 
-version 0.0801_01
+version 0.04
 
 =head1 AUTHOR
 
